@@ -6,6 +6,9 @@ from __future__ import absolute_import, division, print_function
 import numpy as np
 import pysat
 
+import logging
+logger = logging.getLogger(__name__)
+
 
 def download(date_array, tag, sat_id, data_path=None, user=None,
              password=None):
@@ -13,7 +16,7 @@ def download(date_array, tag, sat_id, data_path=None, user=None,
 
     """
     url = 'https://cdpp-archive.cnes.fr/'
-    print('Data must be downloaded by registered users at: {:s}'.format(url))
+    logger.info('Data must be downloaded by registered users at: {:s}'.format(url))
     return
 
 
@@ -37,7 +40,7 @@ def list_remote_files(tag, sat_id):
 
     """
 
-    print('Remote file lists are not supported for Demeter.')
+    logger.info('Remote file lists are not supported for Demeter.')
     return
 
 
@@ -45,12 +48,12 @@ def bytes_to_float(chunk):
     """ Convert a chunk of bytes to a float
 
     Parameters
-    ------------
+    ----------
     chunk : string or bytes
        A chunk of bytes
 
     Returns
-    --------
+    -------
     value : float
         A 32 bit float
 
@@ -75,12 +78,12 @@ def load_general_header(fhandle):
     """ Load the general header block (block 1 for each time)
 
     Parameters
-    ------------
+    ----------
     fhandle : (file handle)
         File handle
 
     Returns
-    ----------
+    -------
     data : list
         List of data values containing: P field,
         Number of days from 01/01/1950, number of miliseconds in the day,
@@ -142,12 +145,12 @@ def load_location_parameters(fhandle):
     time)
 
     Parameters
-    ------------
+    ----------
     fhandle : (file handle)
         File handle
 
     Returns
-    ----------
+    -------
     data : list
         List of data values containing: geoc lat, geoc lon, alt, lt, geom
         lat, geom lon, mlt, inv lat, L-shell, geoc lat of conj point, geoc
@@ -231,12 +234,12 @@ def load_attitude_parameters(fhandle):
     """ Load the attitude parameter block (block 1 for each time)
 
     Parameters
-    ------------
+    ----------
     fhandle : (file handle)
         File handle
 
     Returns
-    ----------
+    -------
     data : list
         list of data values containing: matrix elements from satellite coord
         system to geographic coordinate system, matrix elements from geographic
@@ -297,14 +300,14 @@ def load_binary_file(fname, load_experiment_data):
     """ Load the binary data from a DEMETER file
 
     Parameters
-    ------------
+    ----------
     fname : string
         Filename
     load_experiment_data : function
         Function to load experiment data, taking the file handle as input
 
     Returns
-    ----------
+    -------
     data : np.array
         Data from file stored in a numpy array
     meta : dict
@@ -359,7 +362,7 @@ def set_metadata(name, meta_dict):
     metadata
 
     Parameters
-    ------------
+    ----------
     name : string
         DEMETER instrument name
     meta_dict : dict
@@ -367,7 +370,7 @@ def set_metadata(name, meta_dict):
         attributes are available in the keys 'data names' and 'data units'
 
     Returns
-    ----------
+    -------
     meta : pysat.Meta
         Meta class boject
 
@@ -560,7 +563,7 @@ def set_metadata(name, meta_dict):
                          'satellite_potential': 'Satellite potential'}}
 
     if name not in long_inst.keys():
-        print('Warning, no long-form names available for {:s}'.format(name))
+        logger.warning('no long-form names available for {:s}'.format(name))
 
         long_inst[name] = {nn: nn for nn in meta_dict['data names']}
 
