@@ -1,6 +1,6 @@
 """
 pysat.utils.time - date and time operations in pysat
-=========================================
+====================================================
 
 pysat.utils.time contains a number of functions used throughout
 the pysat package, including interactions with datetime objects,
@@ -128,7 +128,11 @@ def season_date_range(start, stop, freq='D'):
     """
     Deprecated Function, will be removed in future version.
 
-    Replaced by create_date_range
+    .. deprecated:: 2.1.0
+      `season_date_range` will be removed in pysat 3.0.0, this will be
+      replaced by create_date_range
+
+
 
     """
 
@@ -138,7 +142,7 @@ def season_date_range(start, stop, freq='D'):
                             "utils.time.create_date_range instead"]),
                   DeprecationWarning, stacklevel=2)
 
-    season = create_date_range(start, stop, freq='D')
+    season = create_date_range(start, stop, freq=freq)
 
     return season
 
